from math import pi
from casadi import *
import numpy as np
from Layout import Layout
from Model import Model
from collections import OrderedDict
from helpers import casadi_vec, casadi_struct, casadi_vec2struct, casadi_struct2vec


def Initial(p,z,V_block):
	[f,dx,obj,x,z,u,t_end] = Model(p);
	[Int,dae,dae,dae_x,dae_z,dae_u,tau_root,dae_t_end] = Layout(p,f,dx,obj,x,z,u,t_end);
	init = []
	u = [p.P2,p.h,p.W]
	U_init = u
	# ts = ([0] + collocation_points(p.d, 'radau') )
	# t = [x/p.N for x in ts]
	# print t
	# t = [0] + collocation_points(p.d, 'radau')
	for i in range(p.N):

		if i == 0:
			x = [p.Pr_0,p.eta0]
			X_init = x
				
		# algebraic equations
		# Resevoir 
		eq27 = z[26] - z[25] - (p.Pa_Bar)*(z[3]*p.g*p.Delta_h*z[25])/ \
					((z[0]*p.Rg*p.Tr/p.M) + p.P_max*z[25]*(z[1]/(p.rho_w*p.SG_o) \
					+ z[2]/p.rho_w)) # Pressure drop from reservoir to seabed [bar]
		eq28 = z[1] - (p.ko/p.m_max)*((p.P_max*x[0])**2 - (p.P_max*z[25])**2)**p.n # Mass flow of oil in stream 1 [-]
		eq29 = z[2] - (p.kw/p.m_max)*((p.P_max*x[0])**2 - (p.P_max*z[25])**2)**p.n # Mass flow of water in stream 1 [-]
		eq30 = z[0] - z[1]*(p.kg/p.ko)*(p.P_max*x[0] - p.P_max*z[25])**2 # Mass flow of gas in stream 1 [-]
		eq31 = z[3] - z[0] - z[1] - z[2] # Total mass flow in stream 1 [-]
		eq32 = z[9] - z[0]/z[3] # Mass fraction of gas in stream 1
		eq33 = z[10] - z[1]/z[3] # Mass fraction of oil in stream 1
		eq34 = z[11] - z[2]/z[3] # Mass fraction of water in stream 1

		# Valve1	
		eq1 = z[3] - z[4] # constant mass flow over valve1
		eq2 = z[4] - (p.u1*p.Cv1/p.m_max)*sqrt((p.P_max*(z[26]-u[0]))/(z[9]*p.SG_g \
					+ z[10]*p.SG_o + z[11]*p.SG_w)) # Mass flow out of the valve [kt/tau]
		eq3 = z[9] - z[12] # Mass fraction constant over valve
		eq4 = z[10] - z[13] # Mass fraction constant over valve
		eq5 = z[11] - z[14] # Mass fraction constant over valve
		

		# Separator
		eq6 = z[4] - z[5] - z[7] # total mass balance over the separator
		eq7 = z[12]*z[4] - z[15]*z[5] - z[21]*z[7]  # mass balance gass
		eq8 = z[13]*z[4] - z[16]*z[5] - z[22]*z[7] # mass balance oil
		eq9 = z[22] - z[13]*exp(-p.Sep_o*((p.V-z[30]*p.V_max)/ \
			((p.m_max*z[7])/(p.rho_max*z[32])))) # Mass fraction of oil as function of separator efficiecny and resident time.
		eq10 = z[15] - z[12]*exp(-p.Sep_g*((z[30]*p.V_max)/ \
			((p.m_max*z[5])/(p.rho_max*z[31])))) # Mass fraction of gas as function of separator efficiecny and resident time.
		eq11 = z[21] + z[22]  - 1 # mass fractions equal to 1 for stream 5 
		eq12 = z[15] + z[16] + z[17] - 1 # mass fractions equal to 1 for stream 3
		eq13 = u[0] - z[29] # No pressure drop over the separator
		eq14 = z[27] - u[0] - (p.rho_max/p.P_max)*z[31]*p.g*u[1]*(p.Pa_Bar) # Pressure in outlet stream 3 [bar]
		eq15 = z[31] - (p.rho_w/p.rho_max)*( z[15]*p.SG_g + z[16]*p.SG_o + z[17]*p.SG_w) # Density stream 3 [kt/m^3]
		eq16 = z[32] - (p.rho_w/p.rho_max)*(z[21]*p.SG_g + z[22]*p.SG_o )
		eq17 = z[30] - p.L*((u[1] - p.R)*sqrt(2*p.R*u[1]-u[1]**2) \
				+ p.R**2*acos(1-u[1]/p.R))*(1/p.V_max) # Volume of liquid in the separator [m^3]


		# Valve2 
		eq18 = z[5] - z[6] # Mass balance over valve [kt/tau]
		eq19 = z[6] - (p.u1*p.Cv1/p.m_max)*sqrt((p.P_max*(z[27]-z[28]))/(z[15]*p.SG_g \
				+ z[16]*p.SG_o + z[17]*p.SG_w)) # Mass flow out of the valve
		eq20 = z[15] - z[18] # Mass fraction constant over valve
		eq21 = z[16] - z[19] # Mass fraction constant over valve
		eq22 = z[17] - z[20] # Mass fraction constant over valve
		
			
		# Compressor
		eq23 = z[7] - z[8] # mass flow constant over compressor
		eq24 = z[21] - z[23] # Mass fraction constant over compressor
		eq25 = z[22] - z[24] # Mass fraction constant over compressor	
		eq26 = u[2] - (p.gamma/(p.gamma - 1))*((p.Rg*p.Ts/p.M)*((z[33]/z[29])**((p.gamma-1)/p.gamma) \
				-1))*((z[7]*p.m_max)/(p.W_max*x[1]))*(1.1574e-6)  # Compressor duty [MW]
						
	
		if i == 0:
			Sol_z = vertcat(eq30,eq28,eq29,eq31,eq1,eq6,eq18,eq7,eq23,
						eq32,eq33,eq34,eq3,eq4,eq5,eq10,eq8,eq12,
						eq20,eq21,eq22,eq11,eq9,eq24,eq25,
						eq27,eq2,eq14,eq19,eq13,
						eq17,eq15,eq16,eq26)

			f_z = Function("f_z",[z],[Sol_z])
			
			Z_0 = [0.843983, 0.0845017, 0.0132034, 0.941688, 0.941688, 0.0632842, 0.0632842, 0.878404, 0.878404,
				0.896245, 0.0897343, 0.014021, 0.896245, 0.0897343, 0.014021, 1.03142e-009, 0.791364, 0.208636, 
				1.03142e-009, 0.791364, 0.208636, 0.960814, 0.0391857, 0.960814, 0.0391857,
				0.564091, 0.501066, 0.351623, 0.350759, 0.35,
				0.708209, 0.920864, 0.707837, 0.70372]	
				
			s = rootfinder("s", "kinsol", f_z)
		
			Z_init = s(Z_0)
			
		# Integrate over the element
		Ik = Int( x0=X_init, z0=Z_init, p=U_init)

		temp_x = casadi_vec(dae_x,1.0,Pr=X_init[0],eta=X_init[1])
		temp_xc = Ik['xf'][:,1:p.d+1]
		temp_z = Ik['zf'][:,1:p.d+1]
		temp_u = casadi_vec(dae_u,1.0,P2=U_init[0],h=U_init[1],Ws=U_init[2])
		init.append(casadi_vec(V_block,0.0,X=temp_x,XC=temp_xc,Z=temp_z,U=temp_u))
		
		# Output for x and z
		xfin = Ik['xf']
		zfin = Ik['zf']
		X_init = xfin[:,p.d]
		Z_init = zfin[:,p.d]
		
		if i == p.N-1:
			init.append(casadi_vec(dae_x,1.0,Pr=X_init[0],eta=X_init[1]))

	init = vertcat(*init)

	return init