from math import pi
from casadi import *
import numpy as np
from collections import OrderedDict
from helpers import casadi_vec, casadi_struct, casadi_vec2struct, casadi_struct2vec

def Layout(p,f,dx,obj,x,z,u,t_end):	

	dae_int = {'x':x, 'z':z, 'p':u, 't':t_end, 'ode':dx, 'alg':f, 'quad':obj}

	ts = ([0] + collocation_points(p.d, 'radau') )
	t = [q/p.N for q in ts]
	opts = {}
	opts['abstol'] = 1e-8 # abs. tolerance
	opts['reltol'] = 1e-8 # rel. tolerance
	opts['linear_solver'] = 'csparse'
	opts['linear_solver_type'] = 'user_defined'
	opts['grid'] = t
	opts['output_t0'] = True
	Int = integrator('Int', 'idas', dae_int, opts)


	# Make dictionary for variables and equations
	dae_x = OrderedDict()
	dae_z = OrderedDict()
	dae_u = OrderedDict()
	dae_t_end = OrderedDict()
	for k in range(x.shape[0]):
		dae_x.update({str(x[k]):x[k]})
	for j in range(z.shape[0]):
		dae_z.update({str(z[j]):z[j]})
	for i in range(u.shape[0]):
		dae_u.update({str(u[i]):u[i]})


	dae = {}
	dae["x"] = casadi_struct2vec(dae_x)
	dae["z"] = casadi_struct2vec(dae_z)
	dae["p"] = casadi_struct2vec(dae_u)
	if p.On == 1:
		dae_t_end.update({'t_end':t_end})
		dae["t_end"] = casadi_struct2vec(dae_t_end);
	dae["ode"] = dx
	dae["alg"] = f
	dae["quad"] = obj

	# Get collocation points
	tau_root = [0] + collocation_points(p.d, 'radau')
	
	return Int,dae,dae,dae_x,dae_z,dae_u,tau_root,dae_t_end