from math import pi
from casadi import *
import numpy as np

def Model(p):
	# End time variables
	t_end = SX.sym("t_end")
	# Algebraic variables 
	# Mass variables 
	m_g = SX.sym("m_g")
	m_o = SX.sym("m_o")
	m_w = SX.sym("m_w")
	m1 = SX.sym("m1")
	m2 = SX.sym("m2")
	m3 = SX.sym("m3")
	m4 = SX.sym("m4")
	m5 = SX.sym("m5")
	m6 = SX.sym("m6")
	# Mass fraction variables
	Z1_g = SX.sym("Z1_g")
	Z1_o = SX.sym("Z1_o")
	Z1_w = SX.sym("Z1_w")
	Z2_g = SX.sym("Z2_g")
	Z2_o = SX.sym("Z2_o")
	Z2_w = SX.sym("Z2_w")
	Z3_g = SX.sym("Z3_g")
	Z3_o = SX.sym("Z3_o")
	Z3_w = SX.sym("Z3_w")
	Z4_g = SX.sym("Z4_g")
	Z4_o = SX.sym("Z4_o")
	Z4_w = SX.sym("Z4_w")
	Z5_g = SX.sym("Z5_g")
	Z5_o = SX.sym("Z5_o")
	Z6_g = SX.sym("Z6_g")
	Z6_o = SX.sym("Z6_o")
	# Pressure
	Pwf = SX.sym("Pwf")
	P1 = SX.sym("P1")
	P3 = SX.sym("P3")
	P4 = SX.sym("P4")
	P5 = SX.sym("P5")
	# Other variables 
	V_l = SX.sym("V_l") # Liquid volume in the separator [m^3]
	rho_3 = SX.sym("rho_3") # Density of stream 3 [kt/m^3]
	rho_5 = SX.sym("rho_5") # Density of stream 5 [kt/m^3]
	P6 = SX.sym("P6") # Pressure stream 6 [bar]
	
	# Collect all the algebraic variables into a vector
	z = vertcat(m_g,m_o,m_w,m1,m2,m3,m4,m5,m6,
				Z1_g,Z1_o,Z1_w,Z2_g,Z2_o,Z2_w,Z3_g,Z3_o,Z3_w,
				Z4_g,Z4_o,Z4_w,Z5_g,Z5_o,Z6_g,Z6_o,
				Pwf,P1,P3,P4,P5,
				V_l,rho_3,rho_5,P6)
	# Differential variables 
	eta = SX.sym("eta")
	Pr = SX.sym("Pr")
	x = vertcat(Pr,eta)
	# Control Variables 
	P2 = SX.sym("P2")
	h = SX.sym("h")
	Ws = SX.sym("Ws") # Work done by compressor [MW]
	u = vertcat(P2,h,Ws)
	
	# algebraic equations
	# Resevoir 
	eq27 = P1 - Pwf - (p.Pa_Bar)*(m1*p.g*p.Delta_h*Pwf)/ \
			(	(m_g*p.Rg*p.Tr/p.M) + p.P_max*Pwf*(m_o/(p.rho_w*p.SG_o) \
			+ m_w/p.rho_w)) # Pressure drop from reservoir to seabed [bar]
	eq28 = m_o - (p.ko/p.m_max)*((p.P_max*Pr)**2 - (p.P_max*Pwf)**2)**p.n # Mass flow of oil in stream 1 [kt/tau]
	eq29 = m_w - (p.kw/p.m_max)*((p.P_max*Pr)**2 - (p.P_max*Pwf)**2)**p.n # Mass flow of water in stream 1 [kt/tau]
	eq30 = m_g - m_o*(p.kg/p.ko)*(p.P_max*Pr - p.P_max*Pwf)**2 # Mass flow of gas in stream 1 [kt/tau]
	eq31 = m1 - m_g - m_o - m_w # Total mass flow in stream 1 [kt/tau]
	eq32 = Z1_g - m_g/m1 # Mass fraction of gas in stream 1
	eq33 = Z1_o - m_o/m1 # Mass fraction of oil in stream 1
	eq34 = Z1_w - m_w/m1 # Mass fraction of water in stream 1
	
	# Valve1
	eq1 = m1 - m2 # constant mass flow over valve1
	eq2 = m2 - p.u1*p.Cv1/p.m_max*sqrt((p.P_max*(P1-P2))/(Z1_g*p.SG_g \
			+ Z1_o*p.SG_o + Z1_w*p.SG_w)) # Mass flow out of the valve [-]
	eq3 = Z1_g - Z2_g # Mass fraction constant over valve
	eq4 = Z1_o - Z2_o # Mass fraction constant over valve
	eq5 = Z1_w - Z2_w # Mass fraction constant over valve

	# Separator
	eq6 = m2 - m3 - m5 # total mass balance over the separator
	eq7 = Z2_g*m2 - Z3_g*m3 - Z5_g*m5 # mass balance gass
	eq8 = Z2_o*m2 - Z3_o*m3 - Z5_o*m5 # mass balance oil
	eq9 = Z5_o - Z2_o*exp(-p.Sep_o*((p.V-V_l*p.V_max)/((p.m_max*m5)/(p.rho_max*rho_5)))) # Mass fraction of oil as function of separator efficiecny and resident time.
	eq10 = Z3_g - Z2_g*exp(-p.Sep_g*((V_l*p.V_max)/((p.m_max*m3)/(p.rho_max*rho_3)))) # Mass fraction of gas as function of separator efficiecny and resident time.
	eq11 = Z5_g + Z5_o  - 1 # mass fractions equal to 1 for stream 5 
	eq12 = Z3_g + Z3_o + Z3_w - 1 # mass fractions equal to 1 for stream 3
	eq13 = P2 - P5 # No pressure drop over the separator
	eq14 = P3 - P2 - (p.rho_max/p.P_max)*rho_3*p.g*h*(p.Pa_Bar) # Pressure in outlet stream 3 [-]
	eq15 = rho_3 - (p.rho_w/p.rho_max)*( Z3_g*p.SG_g + Z3_o*p.SG_o + Z3_w*p.SG_w) # Density stream 3 [-]
	eq16 = rho_5 - (p.rho_w/p.rho_max)*(Z5_g*p.SG_g + Z5_o*p.SG_o ) # Density stream 3 [-]
	eq17 = V_l - p.L*((h - p.R)*sqrt(2*p.R*h-h**2) + p.R**2*acos(1.0-h/p.R))*(1.0/p.V_max) # Volume of liquid in the separator [-]
	
	# Valve2
	eq18 = m3 - m4 # Mass balance over valve [kt/tau]
	eq19 = m4 - (p.u2*p.Cv2/p.m_max)*sqrt((p.P_max*(P3-P4))/(Z3_g*p.SG_g \
			+ Z3_o*p.SG_o + Z3_w*p.SG_w)) # Mass flow out of the valve
	eq20 = Z3_g - Z4_g # Mass fraction constant over valve
	eq21 = Z3_o - Z4_o # Mass fraction constant over valve
	eq22 = Z3_w - Z4_w # Mass fraction constant over valve


	# Compressor
	eq23 = m5 - m6 # mass flow constant over compressor
	eq24 = Z5_g - Z6_g # Mass fraction constant over compressor
	eq25 = Z5_o - Z6_o # Mass fraction constant over compressor
	eq26 = Ws - (p.gamma/(p.gamma - 1))*((p.Rg*p.Ts/p.M)*((P6/P5)**((p.gamma-1)/p.gamma) \
			-1))*((m5*p.m_max)/(p.W_max*p.eta0))*1.1574e-6 # Compressor duty [MW]
	

	# Collect all of the algebraic equations into a vector
	f = vertcat(eq30,eq28,eq29,eq31,eq1,eq6,eq18,eq7,eq23,
					eq32,eq33,eq34,eq3,eq4,eq5,eq10,eq8,eq12,
					eq20,eq21,eq22,eq11,eq9,eq24,eq25,
					eq27,eq2,eq14,eq19,eq13,
					eq17,eq15,eq16,eq26)
	if p.On == 1:
		# Differential equations
		dPrdt = p.TF*t_end*p.C # Pressure decrease 
		detadt = p.TF*t_end*(p.A*m5*p.m_max*Z5_g + p.B*m5*p.m_max*Z5_o) # Change of compressor efficiency 
		dx = vertcat(dPrdt,detadt)
		
		# Objective function
		obj =   - m6*Z6_g  # objective function
	
	else:
		# Differential equations
		dPrdt = p.TF*p.C # Pressure decrease 
		detadt = p.TF*(p.A*m5*p.m_max*Z5_g + p.B*m5*p.m_max*Z5_o) # Change of compressor efficiency 
		dx = vertcat(dPrdt,detadt)
		
		# Objective function
		obj =   - m6*Z6_g # objective function
	return f,dx,obj,x,z,u,t_end