from math import pi

def Parameters():	
	class para():
		"Parameters and initial conditions"
		pass
	p = para()
	# Initial conditions 
	p.Pr_0 = 1.0 # Pressure in the reservior [-]
	p.eta0 = 1.0 # Starting value for compressor efficiency
	# Valve parameters
	p.Cv1 = 5.5 # Valve 1 sizing coeficient [kt/bar^(1/2) d]
	p.Cv2 = 3.5 #  Valve 2 sizing coeficient [kt/bar^(1/2) d]
	p.u1 = 1.0
	p.u2 = 1.0
	# Separator
	p.L = 7.0 # Length of separator [m]
	p.R = 1.5 # Radius of Separator [m]
	p.V = pi*p.L*p.R**2 # Volume of separator [m^3]
	p.Sep_o = 3500.0 # Separator performance constant for oil separation [1/d]
	p.Sep_g = 2000.0 # Separator performance constant for gas separation [1/d]
	# Compressor
	p.gamma = 1.31 # the ratio of heat capacities (methane)
	p.A = -3.0e-6 # [1/t]
	p.B = -3.0e-5 # [1/t]
	p.P6 = 0.4828 # Pressure after the compression [bar]
	# Reservoir
	p.C = -1.5e-4 # Pressure change with respect to time constant 
	p.ko = 0.0160 # [kt/bar d]
	p.kw = 0.0025 # [kt/bar d]
	p.kg = 1.0e-5 # [kt/bar^3 d]
	p.n = 0.5 # dimesionless constant for Fetkovich equation
	p.Delta_h = -2000 # Height from reservoir to the seabed [m]
	# Data
	p.SG_g = 0.7 # Specific gravity gass [-]
	p.SG_o = 0.9 # Specific gravity oil [-]
	p.SG_w = 1.0 # Specific gravity water [-]
	p.rho_w = 1.0e-3 # Density water [kt/m^3]
	p.g = 7.323e10 # Gravitational constant [m/d^2]
	p.Rg = 0.08314 # Gas constant [bar m^3 / K kmole]
	p.Tr = 374.0 # Temperature in the reservoir [K]
	p.Ts = 274.0 # Temperature in the separator [K]
	p.M = 1.604e-5 # Molecular weight methane [kt/kmol]
	p.N = 60 # Number of intergration points
	p.TF = 1825.0 # End time [d]
	p.Pa_Bar = 1.33959e-9 # Convert from kt/m d^2 to bar
	p.d = 5 # Degree of interpolating polynomial
	p.m_max = 45.34 # Maximum production [kt/d]
	p.P_max = 290.0 # Maximum pressure [bar]
	p.V_max = p.V # Maximum volume of liquid in the separator [m^3]
	p.rho_max = 1.0e-3 # Maximum density [kt/m^3]
	p.W_max = 50.0 # Maximum compressor duty [Mw]
	p.P2 = 0.35
	p.h = 1.0
	p.W = 0.45
	p.On = 1 # 1 end time is a variable, 0 end time is fixed
	p.PV = 1 # Present value on/off.
	p.ns = 1000 # Number of simulations 
	p.tx = 2 + p.d*2 # Starting position of the algebraic variables 
	p.tz = p.tx + 34*p.d # Starting position of the Control variables
	return p